%% Initializations
M = 10; % length of the signal vector x
N = 8; % length of the measurement vector y
Q = 5; % length of the instrument vector z
L = 100; % number of paired training examples {x_n,z_n}
P = 200; % number of paired training examples {y_n,z_n}
T = 10; % number of test examples {x_n,y_n}

%% Construct a random  covariance matrix for the concatenated vector [x;y;z]
A = randn(M+N+Q);
CovMat = A * A';

%% Construct a paired training set of signals and instruments
TrainingSet = A * randn(M+N+Q, L);
X = TrainingSet(1:M, :);
Z_x = TrainingSet(M+N+1:M+N+Q, :);

%% Construct a paired training set of measurements and instruments
TrainingSet = A * randn(M+N+Q, P);
Y = TrainingSet(M+1:M+N, :);
Z_y = TrainingSet(M+N+1:M+N+Q, :);

%% Construct the test set of signals x and measurements y
SigMeas = A * randn(M+N+Q, T);
x = SigMeas(1:M, :);
y = SigMeas(M+1:M+N, :);

%% Choose kernel functions for the minimax estimator
K_z = inline('exp(-mean(d_z(:).^2) / (2 * 2^2))', 'd_z');
K_y = inline('exp(-mean(d_y(:).^2) / (2 * 2^2))', 'd_y');

%% Apply the minimax estimator on each of the test measurements
x_hat_MXREG = zeros(M, T);
for i = 1:T
    x_hat_MXREG(:, i) = MinimaxRegretEstimator(y(:, i), X, Z_x, Y, Z_y, K_z, K_y);
end
% The MSE attained by the minimax regret estimator
MSE_MXREG = mean(sum((x - x_hat_MXREG).^2))

%% Compute the MSE of the MMSE estimator (which has a closed form expression in this case)
Gxx = CovMat(1:M, 1:M);
Gxy = CovMat(1:M, M+1:M+N);
Gyy = CovMat(M+1:M+N, M+1:M+N);
% The MSE attained by the MMSE estimator
MSE_MMSE = trace(Gxx - Gxy * inv(Gyy) * Gxy')